<?php

namespace Drupal\webform_protected_downloads\Plugin\WebformHandler;

use Drupal\Core\Form\FormStateInterface;
use Drupal\file\FileInterface;
use Drupal\file\FileStorageInterface;
use Drupal\file\FileUsage\FileUsageInterface;
use Drupal\webform\Annotation\WebformHandler;
use Drupal\webform\Plugin\WebformHandlerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Handler for protected downloads
 *
 * @WebformHandler(
 *   id = "webform_protected_downloads",
 *   label = @Translation("Webform Protected Download"),
 *   category = @Translation("Downloads"),
 *   description = @Translation("Provides an access controlled link to a file/page of files."),
 *   cardinality = \Drupal\webform\Plugin\WebformHandlerInterface::CARDINALITY_UNLIMITED,
 *   results = \Drupal\webform\Plugin\WebformHandlerInterface::RESULTS_IGNORED,
 *   submission = \Drupal\webform\Plugin\WebformHandlerInterface::SUBMISSION_REQUIRED,
 * )
 */

class WebformProtectedDownloadsHandler extends WebformHandlerBase {

  /**
   * @var FileStorageInterface
   */
  protected $fileStorage;

  /**
   * @var FileUsageInterface
   */
  protected $fileUsage;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $entityTypeManager = $container->get('entity_type.manager');
    $instance->fileStorage = $entityTypeManager->getStorage('file');
    $instance->fileUsage = $container->get('file.usage');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    return [
      'expiration_one_time' => false,
      'expiration_time' => 0,
      'expiration_page' => 'page_reload',
      'expiration_page_custom' => '',
      'expiration_error_message' => 'This link has expired.',
      'protected_file' => [],
      'protected_file_allowed_extensions' => 'gif png jpg jpeg',
      'debug' => false,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $configuration = $this->getConfiguration();
    $settings = $configuration['settings'];

    return [
        '#settings' => $settings,
      ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    // Expiration
    $form['expiration'] = [
      '#type' => 'details',
      '#title' => $this->t('Expiration'),
      '#open' => TRUE
    ];
    $form['expiration']['expiration_onetime'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('One time visit link'),
      '#default_value' => $this->getSetting('expiration_onetime')
    ];
    $form['expiration']['expiration_time'] = [
      '#type' => 'number',
      '#title' => $this->t('Expire after X minutes'),
      '#default_value' => $this->getSetting('expiration_time'),
      '#required' => TRUE,
      '#description' => 'Set this to 0 for unlimited.',
    ];
    $options = [
      '404' => $this->t('404 page'),
      'homepage' => $this->t('Homepage with error message'),
      'page_reload' => $this->t('Form page with error message'),
      'custom' => $this->t('Custom page'),
    ];
    $form['expiration']['expiration_page'] = [
      '#type' => 'radios',
      '#title' => $this->t('Link expired page'),
      '#required' => TRUE,
      '#description' => t('Select a page to be routed when link expires.'),
      '#options' => $options,
      '#default_value' => $this->getSetting('expiration_page'),
      '#attributes' => [
        'name' => 'field_expiration_page',
      ]
    ];
    $form['expiration']['expiration_page_custom'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom link page'),
      '#default_value' => $this->getSetting('expiration_page_custom'),
      '#states' => [
        'visible' => [
          ':input[name="field_expiration_page"]' => [
            'value' => 'custom',
          ]
        ],
        'required' => [
          ':input[name="field_expiration_page"]' => [
            'value' => 'custom',
          ]
        ],
      ],
    ];
    $form['expiration']['expiration_error_message'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Error message'),
      '#description' => $this->t('Error message to display.'),
      '#default_value' => $this->getSetting('expiration_error_message'),
      '#states' => [
        'visible' => [
          ':input[name="field_expiration_page"]' => [
            ['value' => 'homepage'],
            ['value' => 'page_reload'],
          ],
        ],
        'required' => [
          ':input[name="field_expiration_page"]' => [
            ['value' => 'homepage'],
            ['value' => 'page_reload'],
          ],
        ]
      ],
    );

    // Files
    $form['protected'] = [
      '#type' => 'details',
      '#title' => $this->t('Protected Files'),
      '#open' => TRUE
    ];
    $form['protected']['protected_file'] = [
      '#name' => 'protected_file',
      '#type' => 'managed_file',
      '#title' => $this->t('Choose file for protected download'),
      '#multiple' => FALSE,
      '#upload_validators'  => [
        'file_validate_extensions' => [$this->getSetting('protected_file_allowed_extensions')]
      ],
      '#upload_location' => 'private://webform_protected_downloads/[date:custom:Y]-[date:custom:m]',
      '#default_value' => $this->getSetting('protected_file'),
    ];
    $form['protected']['protected_file_allowed_extensions'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Valid File extensions'),
      '#description' => $this->t("Separate extensions with a space."),
      '#default_value' => $this->getSetting('protected_file_allowed_extensions'),
      '#required' => TRUE,
    ];

    // Development.
    $form['development'] = [
      '#type' => 'details',
      '#title' => $this->t('Development settings'),
      '#open' => true
    ];
    $form['development']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debugging'),
      '#description' => $this->t('If checked, debugging will be displayed onscreen to all users.'),
      '#return_value' => true,
      '#default_value' => $this->getSetting('debug'),
    ];

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    $this->configuration = $this->defaultConfiguration();
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);

    // Set the files as permanent through file usage
    $protectedFiles = $form_state->getValue('protected_file');
    foreach ($protectedFiles as $protectedFile) {
      /** @var FileInterface $file */
      $file = $this->fileStorage->load($protectedFile);
      if ( ! empty($file)) {
        $this->fileUsage->add($file,'webform_protected_downloads', 'webform', $this->getWebform()->id());
      }
    }
  }

}
